/*
 * Copyright (c) 1997, 1999, 2000, Mark Buser.
 * Copyright (c) 2001, 2003, 2004, Danny Backx.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the names the authors (see above), nor the names of other
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Header: /pack/anoncvs/xinvest/src/optdetail.c,v 1.28 2004/04/30 16:12:33 danny Exp $
 */
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <Xm/XmAll.h>
#include <Xbae/Matrix.h>

#include "fileXq.h"
#include "optnet.h"
#include "optdetail.h"
#include "opttick.h"
#include "pixmap.h"
#include "server.h"
#include "status.h"
#include "tape.h"
#include "xquote.h"
#include "xutil.h"

/* Globals */
static Widget checkbox;  /* toggles of current state */
static int state = 0;    /* state of detail toggle buttons */



/*
 * This array holds the data to be displayed.
 */
char	***values = NULL;
int	v_nrows = 0, v_ncols = 0;

/*
 * The dimensions of the XbaeMatrix and the 'helper' arrays.
 */
extern Widget	Container;
int		nrows = 0, ncols = -1;
short	*column_widths = NULL;
char	**column_labels = NULL;
static int	xbae_field[NUM_QUERY_VALUES];
static int	xbae_nfields = 0;

/*
** Detail option support
*/

static void ChangeState(int which, int nfields)
{
	int	i;

	if (! column_widths)
		column_widths = (short *)XtCalloc(sizeof(short), NUM_QUERY_VALUES+1);

	if (ncols < 0) {
		XtVaGetValues(Container, XmNcolumns, &ncols, NULL);
	}

	/*
	 * NFIELDS is the number of user-selected fields.
	 * NCOLS is the number of columns in the XbaeMatrix.
	 * NCOLS should be NFIELS+2 to account for the up/down arrows and the ticker
	 */
	if (nfields + 2 != ncols) {
		/* Change XbaeMatrix #columns */
		column_widths[0] = 3;	/* special case, for the pixmaps */
		for (i=1; i<=NUM_QUERY_VALUES; i++)
			column_widths[i] = 1;

		if (nfields+2 < ncols) {
			/* Remove columns */
			int	d = -(nfields + 2 - ncols);
			XbaeMatrixDeleteColumns(Container, 0, d);
			ncols -= d;
		} else if (ncols < nfields+2) {
			/* Add columns */
			short	*wp = NULL;
			int	d = nfields + 2 - ncols;

			wp = (short *)XtCalloc(d, sizeof(short));
			for (i=0; i<d; i++)
				wp[i] = 3;
			XbaeMatrixAddColumns(Container, 0, NULL, NULL, wp, NULL,
				NULL, NULL, NULL, d);
			ncols += d;
			xbae_nfields += d;
			XtFree((char *)wp);

#if 0
			FreeValuesArray();
#endif
		}

		state = (which == 0) ? 1 : 0;	/* To make the test below succeed */
	}
	if (state != which) {
		/* Other fields to display */
		/* FIX ME */
	}

	state = which;
}

static void ChangeNumRows(int count)
{
	int	i;

	if (v_nrows <= count) {
		i = v_nrows;
		v_nrows = count;
		values = (char ***)XtRealloc((char *)values,
					sizeof(char **) * (v_nrows));
		for (; i < v_nrows; i++)
			values[i] = (char **)XtCalloc(NUM_QUERY_VALUES, sizeof(char *));
	}

	if (! column_labels) {
		column_labels = (char **) XtCalloc(sizeof(char *), NUM_QUERY_VALUES + 1);
		for (i=0; i<NUM_QUERY_VALUES+1; i++) {
			column_labels[i] = strdup("");
		}
	}
	if (! column_widths) {
		column_widths = (short *)XtCalloc(sizeof(short), NUM_QUERY_VALUES + 1);
		column_widths[0] = 3;
		for (i=0; i<NUM_QUERY_VALUES+1; i++) {
			column_widths[i] = 5;	/* FIX ME */
		}
	}

	nrows = count;
	XtVaSetValues(Container,
		XmNvisibleRows,			50,
		XmNrows,			nrows,
		XmNcolumns,			ncols,
		XmNrowLabels,			NULL,
		XmNcolumnLabels,		column_labels,
		XmNcolumnWidths,		column_widths,
		XmNcolumnAlignments,		NULL,
		XmNcolumnLabelAlignments,	NULL,
		XmNcolumnMaxLengths,		NULL,
		XmNcolumnFontBold,		NULL,
		XmNcolumnButtonLabels,		NULL,
		XmNshowColumnArrows,		NULL,
		NULL);
}

/* ARGSUSED */
void popupDetailCB( Widget w, XtPointer menu_pos, XtPointer call_data)
{
  int ticknum;
  char *url;

  switch ((int)menu_pos) {

    case 0: /* Copy URL to clipboard  */
	    XtVaGetValues (XtParent(w), XmNuserData, &ticknum, NULL);
	    /* If this isn't a valid ticker, bail */
	    if ((ticknum<0) || (ticknum>tickGetNum())) 
	      break;

	    url = makeQuery (1, ticknum);
	    url = strtok (url, " ");       /* Skip GET */
	    url = strtok (NULL, " ");      /* Truncate at end of URL */

	    /* First the Motif way using the CLIPBOARD */
	    {
	      int status;
	      long item_id;
	      XmString labelXm;

	      labelXm = XmStringCreateLocalized ("Xquote");
	      do {
		status = XmClipboardStartCopy (XtDisplayOfObject(w), 
				             XtWindowOfObject(w), labelXm,
					     CurrentTime, NULL, NULL, &item_id);
	      } while (status == ClipboardLocked);
	      XmStringFree (labelXm);

	      do {
		status = XmClipboardCopy (XtDisplayOfObject(w), 
				          XtWindowOfObject(w), item_id, 
					  "STRING", url, strlen(url)+1,
					  0L, NULL);
	      } while (status == ClipboardLocked);
	       
	      do {
		status = XmClipboardEndCopy (XtDisplayOfObject(w), 
				             XtWindowOfObject(w), item_id);
	      } while (status == ClipboardLocked);
	    }

	    /* Now the Xt way using the PRIMARY selection */
	    break;

    default:
	    break;
  }
}

/*
 * This function gets called by the Details dialog.
 * We're now supposed to figure out which fields the user wants to display.
 */
/* Set which detail variables are selected. */
void detailSetState( int which )
{
  WidgetList children;
  int numChild = 0;
  int i, j;

#undef	XX_1
  if (checkbox)
    XtVaGetValues ( checkbox, XmNchildren, &children,
  		              XmNnumChildren, &numChild,
  		    NULL);
#ifdef	XX_1
  fprintf(stderr, "Fields [");
#endif
  for (i=0, xbae_nfields=0; i < numChild; i++) {
    XmToggleButtonSetState ( children[i], 
		             (which & (1 << i))?XmSET:XmUNSET, 
			     False);
    if (which & (1 <<i)) {
#ifdef	XX_1
	fprintf(stderr, "%2d ", xbae_nfields);
#endif
	xbae_field[i] = xbae_nfields++;
    } else {
#ifdef	XX_1
	fprintf(stderr, "xx ");
#endif
	xbae_field[i] = -1;
    }
  }
#ifdef	XX_1
  fprintf(stderr, "]\n");
#endif
  ChangeState(which, xbae_nfields);
}

/* Non-zero if state variable 'which' is selected, otherwise entire state. */
int detailGetState( int which )
{
  if (which == -1)
    return (state);
  else
    return (state & (1 << which));
}

void DrawCellCB(Widget w, XtPointer client, XtPointer call)
{
	XbaeMatrixDrawCellCallbackStruct *cbp = (XbaeMatrixDrawCellCallbackStruct *)call;

	if (cbp->row >= v_nrows || cbp->column >= v_ncols)
		return;

	if (cbp->column == 0) {
		cbp->pixmap = (Pixmap)XbaeMatrixGetCellUserData(Container, cbp->row, cbp->column);
		if (cbp->pixmap) {
			cbp->type = XbaePixmap;
			cbp->width = cbp->height = 16;
		} else {
			cbp->type = XbaeString;
			cbp->string = "";
		}
	} else {
		cbp->type = XbaeString;
		cbp->string = values ? values[cbp->row][cbp->column] : "";
	}
}

static void FreeValuesArray(void)
{
	int	i, j;

	if (values == NULL)
		return;
	for (i=0; i<v_nrows; i++) {
		for (j=0; j<v_ncols; j++) {
			if (values[i][j])
				XtFree(values[i][j]);
		}
		XtFree((char *)values[i]);
	}
	XtFree((char *)values);
	values = NULL;
}

/* Change main window detail values to match net data */
void detailUpdateMainWindow (int refresh)
{
  extern Widget Container, Scroll;
  Widget NewContainer, popupDetail;

  Widget Item, Pix, w;
  char name[12];
  char * TickString;
  XmString details[NUM_QUERY_VALUES];
  int num_details = 0;
  Pixmap pixmap;

  FILE *fp = getLogFilePointer();     /* File to log to if enabled */
  int fFormat = getLogFileFormat();   /* Format to log with        */
  int some_data, nrows, ncols;
  Widget p = XtParent(Container);
  int	do_resize = 0;

  int i, j, count;

  if (!Container)
    return;

  if (refresh == DETAIL_EDIT)         /* On detail option change, don't log */
    fp = 0;

  AllowShellResize (per->Toplevel, ALLOW);

#ifdef	TOGGLE_REDISPLAY
  XbaeMatrixDisableRedisplay(Container);
#endif

  count = tickGetNum();
  XtVaGetValues(Container, XmNrows, &nrows, XmNcolumns, &ncols, NULL);
  if (! values) {
    /* Allocate array */
    v_nrows = (count < 5) ? 5 : count;
    values = (char ***)XtCalloc(v_nrows, sizeof(char **));
    for (i=0; i<v_nrows; i++) {
      values[i] = (char **)XtCalloc(NUM_QUERY_VALUES, sizeof(char *));
    }
    v_ncols = xbae_nfields + 2;
  }
  ChangeState(state, ncols - 2);
  ChangeNumRows((count < 5) ? 5 : count);

  for (i=0; i < count; i++) {
    /*
    ** Fill in detail information.
    */
    QUERY_STRUCT *data;

    data = tickGetDetail(i);
    TickString = tickGetNickName(i) ? tickGetNickName(i) : tickGetName(i);
#if 0
fprintf(stderr, "SetCell(%d,%d,%s)\n", i, 0, TickString);
#endif
    XbaeMatrixSetCell(Container, i, 1, TickString);
    values[i][0] = NULL;
    values[i][1] = strdup(TickString);

    if (column_widths[1] < strlen(TickString))
		column_widths[1] = strlen(TickString) + 1;

    if (!data || data->values[CHANGE] == NULL) {
      pixmap = GetPixmap( PERROR, NORMAL, XtDisplay(per->Toplevel));
    } else if (strchr (data->values[CHANGE], '-')) {
      pixmap = GetPixmap( PDOWN, NORMAL, XtDisplay(per->Toplevel));
    } else {
      pixmap = GetPixmap( PUP, NORMAL, XtDisplay(per->Toplevel));
    }
    XbaeMatrixSetCellUserData(Container, i, 0, (XtPointer)pixmap);	/* Draw this pixmap */

    /* Loop over selected detail info filling it in */
    num_details = 0;
    some_data = 0;

    for (j=PRICE; j < NUM_QUERY_VALUES; j++) {
      char tmp[13] = "N/A";

      if ( detailGetState(j-PRICE) ) {
        if (data && data->values[j]) {
	  if ( j == VOLUME || j == EXDIV || j == DATE || j == TIME || 
	       j == NAME )
	    sprintf ( tmp, "%-.12s", data->values[j]);
	  else
	    sprintf ( tmp, "%.12s", data->values[j]);
	  some_data = 1;
	}
	if (xbae_field[j-2] >= 0) {
#if 0
/* if (i == 0)fprintf(stderr, "SetCell(%d,%d -> %d,%s)\n", i, j, 2+xbae_field[j-2], tmp); */
fprintf(stderr, "SetCell(%d,%d -> %d,%s)\n", i, j, 2+xbae_field[j-2], tmp);
#endif
		XbaeMatrixSetCell(Container, i, 2+xbae_field[j-2], tmp);
		values[i][2+xbae_field[j-2]] = strdup(tmp);
	}
	if (column_widths[2+xbae_field[j-2]] < strlen(tmp))
		column_widths[2+xbae_field[j-2]] = strlen(tmp) + 1;

	/* 
	** File Logging
	*/
        if (fp) { /* Log to file */
	  char datebuf[40];
          time_t t = time ((time_t *)NULL);
          struct tm *today = localtime (&t);

          if (fFormat) { /* Xinvest format */
            strftime (datebuf, 40, "%m/%d/%Y", today);
	    if (j==PRICE && strcmp(tmp, "N/A") != 0)
              fprintf (fp, "# %s\n%s\tPrice\t%s", tickGetName(i), datebuf, tmp);

	  } else {       /* CSV format */
            strftime (datebuf, 40, "%c", today);
	    if (j==PRICE)
              fprintf (fp, "%s; %s; ", datebuf, tickGetName(i));
            fprintf (fp, "%s; ", tmp);
	  }
	}

      } /* if detail on */
    } /* for all details */
    if (fp && some_data) {
      fprintf (fp, "\n");
      fflush (fp);
    }
  }
	for (i=1; i < ncols; i++) {
		if (column_widths[i] < strlen(column_labels[i])+1)
			column_widths[i] = strlen(column_labels[i])+1;
#if 0
		fprintf(stderr, "Column %d is %d wide(%s)\n", i, column_widths[i], column_labels[i]);
#endif
	}
  XtVaSetValues(Container,
	XmNcolumnWidths,	column_widths,
	NULL);

#ifdef	TOGGLE_REDISPLAY
  XbaeMatrixEnableRedisplay(Container, True);
#endif

  /* Show the time of last update */
  {
  time_t t = time ( (time_t *)NULL ); 
  struct tm *now = localtime(&t);
  char line[80];
  strftime(line, sizeof(line), "Last update : %x %X", now);
  write_lastupdate_line(line);
  }
}

void detailSetHeading ( XmStringTable headings, int num_headings )
{
	int i, m;

	if (! column_widths)
		column_widths = (short *)XtCalloc(sizeof(short), NUM_QUERY_VALUES+1);
	if (! column_labels)
		column_labels = (char **)XtCalloc(sizeof(char *), (NUM_QUERY_VALUES + 1));

	column_widths[0] = 3;	/* special case, for the pixmaps */
	for (i=1; i<NUM_QUERY_VALUES+1; i++)
		column_widths[i] = 1;

	ChangeState(state, num_headings-1);

	for (i=0; i<=NUM_QUERY_VALUES; i++)
		column_labels[i] = strdup("");
	for (i=0; i < num_headings; i++) {
		char *s = NULL;
		if (XmStringGetLtoR(headings[i], XmFONTLIST_DEFAULT_TAG, &s)) {
			XbaeMatrixSetColumnLabel(Container, i+1, s);
			if (column_labels[i+1])
				XtFree(column_labels[i+1]);
			column_labels[i+1] = /* strdup */ (s);
			/* XtFree(s); */
			if (column_widths[i+1] < strlen(s))
				column_widths[i+1] = strlen(s) + 1;
#if 0
			fprintf(stderr, "Heading[%d,%s] width %d\n", i+1, s, column_widths[i+1]);
#endif
		}
	}

	XtVaSetValues(Container,
		XmNcolumnWidths,	column_widths,
		NULL);
}

/* Change main window detail list to match state */
void detailAddMainWindow ()
{
  extern Widget checkbox;
  WidgetList children;
  int numChild;

  XmStringTable headings;
  int nh = 0;
  int i;

  AllowShellResize (per->Toplevel, ALLOW);

  /*
  ** Get detail box labels
  */
  XtVaGetValues ( checkbox, XmNchildren, &children,
		            XmNnumChildren, &numChild,
		  NULL);
  headings = (XmStringTable) XtMalloc ( (numChild+1) * sizeof (XmString) );
  if (headings == (XmStringTable) NULL )
    write_status ("Could not allocate memory, detail list not changed.", ERR);

  headings[nh++] =  XmStringCreateLocalized ("Ticker");
  for (i=0; i < numChild; i++) {
    if ( state & (1<<i) )
      XtVaGetValues ( children[i], 
                      XmNlabelString, &(headings[nh++]),
                      NULL);
  }

  /*
  ** Apply them
  */
  if (nh)
    detailSetHeading ( headings, nh );
  XtFree ( (char *)headings);

  /*
  ** Update main window detail info
  */
  detailUpdateMainWindow (DETAIL_EDIT);

  AllowShellResize (per->Toplevel, ALLOW);
}


/* Remember or restore toggle button states */

/* ARGSUSED */
void procDetail (Widget w, XtPointer which, XtPointer call_data)
{
  int oldstate = state;

  int i, numchild, j;
  Widget parent;
  WidgetList children;

  XtVaGetValues ( w, XmNuserData, &parent, NULL);
  XtVaGetValues ( parent, 
                  XmNnumChildren, &numchild,
                  XmNchildren, &children, 
                  NULL);

  switch ((int)which) {

    case 0: /* Ok */
            XtPopdown (GetTopShell(w));

            /* Save toggle settings */
            for (i = 0, xbae_nfields = 0; i < numchild; i++) {
              if (XmToggleButtonGadgetGetState (children[i])) {
                state |= (1<<i);
		xbae_field[i] = xbae_nfields++;
              } else {
                state &= ~(1<<i);
		xbae_field[i] = -1;
	      }
            }

            if (state != oldstate)
              detailAddMainWindow ();

            break;

    case 1: /* Cancel */
            XtPopdown (GetTopShell(w));

            /* Restore toggle settings */
            for (i = 0; i < numchild; i++) {
              if (state & (1<<i))
                XmToggleButtonGadgetSetState (children[i], XmSET, False);
              else
                XmToggleButtonGadgetSetState (children[i], XmUNSET, False);
            }
            break;

    default: break;
  }
}

/* Create the detail dialog, but don't manage it */
Widget createDetailDialog () 
{
  Widget dialog;
  char *buttons[] = { "button_0", "button_1"};
  char name[10];
  Widget form, pane, button;
  Dimension width, height, border;
  int num;

  dialog = XtVaCreatePopupShell( "OptionDetail",
                              xmDialogShellWidgetClass,
                              GetTopShell(per->Toplevel),
			      XmNmappedWhenManaged, False,
                              NULL );
  pane = XtVaCreateWidget ( "DetailPane", xmPanedWindowWidgetClass,
                              dialog,
                              XmNsashWidth, 1,
                              XmNsashHeight, 1,
                              NULL );
  checkbox = XmVaCreateSimpleCheckBox ( pane,"DetailCheckBox", NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
			      XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              XmVaCHECKBUTTON, NULL, NULL, NULL, NULL,
                              NULL );
  XtManageChild (checkbox);

  for (num = 0; num < 14; num++) {
    sprintf (name, "button_%d", num);
    if ( (state & (1 << num)) && (button = XtNameToWidget( checkbox, name )) )
      XmToggleButtonGadgetSetState ( button, True, False);
  }


  /* Buttons to add ok and cancel */
  form = XtVaCreateWidget ( "ButForm", xmFormWidgetClass, pane, NULL );
  for (num=0; num < XtNumber(buttons); num++) {
    button = XtVaCreateManagedWidget ( buttons[num],
                             xmPushButtonWidgetClass, form,
                              XmNtopAttachment, XmATTACH_FORM,
                              XmNbottomAttachment, XmATTACH_FORM,
                              XmNleftAttachment, XmATTACH_POSITION,
                              XmNrightAttachment, XmATTACH_POSITION,
                              XmNshowAsDefault, (num==0)?True:False,
                              XmNdefaultButtonShadowThickness, 1,
                              XmNuserData, checkbox,
                              NULL );
    XtAddCallback ( button, XmNactivateCallback,
                    (XtCallbackProc) procDetail, (XtPointer) num );
  }
  XtManageChild (form);
  XtManageChild (pane);

  /* Prevent pane from changing size */
  XtVaGetValues ( dialog,
                  XmNwidth, &width,
                  XmNheight, &height,
	          XmNborderWidth, &border,
                  NULL );

  XtVaSetValues ( dialog,
                  XmNminWidth,  width +  border,
                  XmNmaxWidth,  width +  border,
                  XmNminHeight, height + border,
                  XmNmaxHeight, height + border,
                  NULL );
 return (dialog);
}
